# frozen_string_literal: true

require 'prolog/dry_types'

module ArticleFixtureGen
  module Data
    # Assembles a "fragment list", a list of objects associating a string as
    # found in a DOM leaf text node with the DOM position of that leaf node
    # relative to a specified parent node and initial position. The default
    # initial position, `[]`, indicates that the parent node is the root node
    # of a DOM tree known to the caller.
    class BuildFragmentList
      def self.call(parent_node:, position: [])
        BuildFragmentList.new.call(parent_node, position)
      end

      def call(parent_node, position)
        ret = []
        Internals.with_each_child_of(parent_node) do |node, index|
          ret += add_items_for(node, index, position)
        end
        ret
      end

      private

      def add_items_for(node, index, position)
        item_pos = Internals.position_with(position, index)
        return call(node, item_pos) if Internals.node?(node)
        Internals.item_with(item_pos, node)
      end

      # Stateless methods.
      module Internals
        def self.item_with(position, string)
          Array(FragmentPosition.new(position: position, string: string))
        end

        # Reek doesn't like .respond_to?; it flags :reek:ManualDispatch. Tough.
        def self.node?(item)
          item.respond_to?(:nodes)
        end

        def self.position_with(position, index)
          position + Array(index)
        end

        # This is here because we need *both* the node and the index, so
        # `Array#map` doesn't do us for.
        def self.with_each_child_of(parent_node)
          parent_node.nodes.each_with_index { |node, index| yield(node, index) }
        end
      end
      private_constant :Internals

      # Associates a DOM-fragment string with a node position. The position is
      # given as an array of indexes relative to the specified parent node.
      # For example, given an object `obj` with :position of [0, 1, 7, 4], then
      #     obj.string == parent_node.nodes[0].nodes[1].nodes[7].nodes[4]
      # would be true.
      class FragmentPosition < Dry::Struct::Value
        attribute :position, Types::Strict::Array.member(Types::Strict::Int)
        attribute :string, Types::Strict::String
      end
      private_constant :FragmentPosition
    end # class ArticleFixtureGen::Data::BuildFragmentList
  end
end
