# frozen_string_literal: true

require 'json'
require 'prolog/dry_types'

require_relative './build_fragment_list'

module ArticleFixtureGen
  module Data
    # Value object associating a "word" (non-whitespace substring of a text leaf
    # node) with a DOM position and an index within that leaf node.
    class WordEntry < Dry::Struct::Value
      attribute :dom_position, Types::Strict::Array.member(Types::Strict::Int)
      attribute :word, Types::Strict::String
      attribute :word_index, Types::Strict::Int.constrained(gteq: 0)

      def <=>(other)
        WordEntryComparator.call(self, other)
      end
    end # class ArticleFixtureGen::Data::WordEntry

    # Compares two WordEntry instances to each other, per comparison interface.
    class WordEntryComparator
      def self.call(obj, other)
        WordEntryComparator.new(obj, other).call
      end

      def call
        Internals.first_nonzero([dom_difference, index_difference]) || 0
        # [dom_difference, index_difference].reject(&:zero?).first
      end

      protected

      def initialize(obj, other)
        @obj = obj
        @other = other
        self
      end

      private

      def dom_difference
        @obj.dom_position <=> @other.dom_position
      end

      def index_difference
        @obj.word_index <=> @other.word_index
      end

      # Stateless methods.
      module Internals
        def self.first_nonzero(values)
          _nonzero_values_for(values).first
        end

        def self._nonzero_values_for(values)
          values.select(&:nonzero?)
        end
      end
      private_constant :Internals
    end # class ArticleFixtureGen::Data::WordEntryComparator

    # Builds an array of WordEntry items, associating fragments of DOM text leaf
    # nodes with position information of their containing node within the DOM
    # and of the fragment within a (whitespace-split) array of fragments within
    # the containing DOM node's text.
    class BuildWordList
      def self.call(parent_node:, dom_position: [])
        BuildWordList.new(parent_node, dom_position).call
      end

      def call
        ret = string_items.map do |string_item|
          Internals.entries_for string_item
        end
        ret.flatten
      end

      protected

      def initialize(parent_node, dom_position)
        @dom_position = dom_position
        @parent_node = parent_node
        self
      end

      private

      attr_reader :dom_position, :parent_node

      def string_items
        BuildFragmentList.call(parent_node: parent_node, position: dom_position)
      end

      # Statelesss methods
      module Internals
        def self.entries_for(string_item)
          EntriesForStringItem.call string_item: string_item
        end
      end
      private_constant :Internals

      # Build an arary of WordEntry instances for "words" in a "string item".
      class EntriesForStringItem
        def self.call(string_item:)
          EntriesForStringItem.new(string_item).call
        end

        def call
          words.map { |word| entry_for word }
        end

        protected

        def initialize(string_item)
          @position = string_item.position
          @word_index = 0
          @words = string_item.string.split
          self
        end

        private

        attr_reader :position, :word_index, :words

        def entry_for(word)
          ret = WordEntry.new entry_params_for(word)
          @word_index += 1
          ret
        end

        def entry_params_for(word)
          { dom_position: position, word: word, word_index: word_index }
        end
      end # class EntriesForStringItem
      private_constant :EntriesForStringItem
    end # class ArticleFixtureGen::Data::BuildWordList
  end
end
