# frozen_string_literal: true

require_relative './build_target_entry_list'
require_relative './marker_array'
require_relative './node_markup'
require_relative './replace_child_nodes_using_new_child'
require_relative './split_text_at_target_word'

module ArticleFixtureGen
  module Data
    # Shared logic for marker tag pairs, both single MTPs and paired MTPs. This
    # has been reworked so that the `attributes` parameter specifies the number
    # of marker-tag pairs to be generated, by specifying the ID attributes of
    # each MTP to be generated, working from the end of the content toward the
    # beginning. From the standpoint of this class and its collaborators, doing
    # so eliminates any need to differentiate between sMTPs and pMTPs, since
    # each marker-tag pair is generated based on passed-in information.
    class MtpDecoratedMarkup
      def self.call(attributes:, base_markup:)
        new(attributes).call(base_markup)
      end

      def call(base_markup)
        parent_node = NodeFromMarkup.call base_markup
        add_entries(parent_node)
        MarkupFromNode.call parent_node
      end

      protected

      def initialize(attributes)
        @attributes = attributes
        @markers = build_markers
        self
      end

      private

      attr_reader :attributes, :markers

      def add_entries(parent_node)
        each_entry_for(parent_node) do |target_entry, index|
          add_marker(parent_node, target_entry, markers[index])
        end
      end

      def add_marker(parent_node, target_entry, marker)
        ReplaceChildNodesUsingNewChild.call(parent_node, target_entry, marker)
        self
      end

      def build_markers
        MarkerArray.call attributes: attributes
      end

      def each_entry_for(parent_node)
        target_entries(parent_node).each_with_index do |target_entry, index|
          yield target_entry, index
        end
      end

      def target_entries(parent_node)
        BuildTargetEntryList.call(parent_node, attributes.count)
      end
    end # class ArticleFixtureGen::Data::MtpDecoratedMarkup
  end
end
